@@ -71,9 +71,28 @@ do_ndsctl () {
 #					#
 #########################################
 
+# Get tmpfs mountpoint
+mountpoint=$(/usr/lib/opennds/libopennds.sh tmpfs)
+
+# Clean up any previous ndsctl debuglevel setting
+if [ -e "$mountpoint/ndsdebuglevel" ]; then
+	rm $mountpoint/ndsdebuglevel
+fi
+
 #wait a while for openNDS to get started
 sleep 5
 
+# Get status as ndsctlout
+ndsctlcmd="status 2>/dev/null"
+do_ndsctl
+
+# Get the configured debuglevel and version
+debuglevel=$(echo "$ndsctlout" | grep "Debug Level" | awk '{printf $4}')
+version=$(echo "$ndsctlout" | grep "Version" | awk '{printf $2}')
+echo "debuglevel=$debuglevel"
+# Get PID For syslog
+ndspid=$(pgrep '/usr/bin/opennds')
+
 #get arguments and set variables
 url=$1
 gatewayhash=$2
@@ -82,11 +101,21 @@ loopinterval=10
 postrequest="/usr/lib/opennds/post-request.php"
 
 # Construct our user agent string:
-ndsctlcmd="status 2>/dev/null"
-do_ndsctl
-version=$(echo "$ndsctlout" | grep Version | awk '{printf $2}')
 user_agent="openNDS(authmon;NDS:$version;)"
 
+# If we are on OpenWrt, check if ca-bundle is installed
+owrt=$(type "opkg" 2>/dev/null | grep "/")
+
+if [ ! -z "$owrt" ]; then
+	cabundle=$(opkg list-installed | grep "ca-bundle")
+
+	if [ -z "$cabundle" ]; then
+		echo "authmon - FATAL ERROR: ca-bundle not installed - Terminating" | logger -p "daemon.err" -s -t "opennds[$ndspid]"
+		ndsctl stop
+		exit 1
+	fi
+fi
+
 # Call postrequest with action.
 # Action can be "list" (list and delete from FAS auth log), "view" (view and leave in FAS auth log) or "clear" (clear any stale FAS auth log entries)
 # Initialise by clearing stale FAS auth log entries
@@ -94,25 +123,27 @@ action="clear"
 payload="none"
 ret=$($phpcli -f "$postrequest" "$url" "$action" "$gatewayhash" "$user_agent" "$payload")
 
-# Get the configured debuglevel
-option="debuglevel"
-get_option_from_config
-
-if [ -z "$debuglevel" ]; then
-	$debuglevel="1"
-fi
-
-# Get PID For syslog
-ndspid=$(pgrep '/usr/bin/opennds')
 
 # Main loop:
 while true; do
+	# Check if debuglevel has been changed by ndsctl
+
+	if [ -e "$mountpoint/ndsdebuglevel" ]; then
+		debuglevel=$(cat $mountpoint/ndsdebuglevel)
+		echo "debuglevel=$debuglevel"
+	fi
+
 	# Get remote authlist from the FAS:
 	action="view"
 	payload="none"
 	acklist="*"
 
 	authlist=$($phpcli -f "$postrequest" "$url" "$action" "$gatewayhash" "$user_agent" "$payload")
+
+	if [ $debuglevel -ge 3 ]; then
+		echo "authmon - authlist $authlist" | logger -p "daemon.debug" -s -t "opennds[$ndspid]"
+	fi
+
 	validator=${authlist:0:1}
 
 	if [ "$validator" = "*" ]; then
@@ -127,8 +158,8 @@ while true; do
 			for authparams_enc in $authlist; do
 				authparams=$(printf "${authparams_enc//%/\\x}")
 
-				if [ "$debuglevel" -ge 2 ]; then
-					echo "authmon - authentication parameters $authparams" | logger -p "daemon.info" -s -t "opennds[$ndspid]: "
+				if [ $debuglevel -ge 2 ]; then
+					echo "authmon - authentication parameters $authparams" | logger -p "daemon.info" -s -t "opennds[$ndspid]"
 				fi
 
 				ndsctlcmd="auth $authparams 2>/dev/null"
@@ -141,7 +172,7 @@ while true; do
 				fi
 
 				if [ "$ndsstatus" = "busy" ]; then
-					logger -s -p daemon.err -t "authmon" "ERROR: ndsctl is in use by another process"
+					echo "authmon - ERROR: ndsctl is in use by another process" | logger -p "daemon.err" -s -t "opennds[$ndspid]"
 				fi
 
 				if [ "$authcount" < 1 ]; then
@@ -153,7 +184,9 @@ while true; do
 		for keyword in $authlist; do
 
 			if [ $keyword = "ERROR:" ]; then
-				logger -s -p daemon.err -t "authmon" "[$authlist]"
+				echo "authmon - [$authlist]" | logger -p "daemon.err" -s -t "opennds[$ndspid]"
+				echo "authmon - Check Internet connection, FAS url, and package ca-bundle installation" |
+					logger -p "daemon.err" -s -t "opennds[$ndspid]"
 				break
 			fi
 		done
@@ -162,6 +195,9 @@ while true; do
 	# acklist is a space separated list of the rhid's of sucessfully authenticated clients.
 	# Send acklist to the FAS for upstream processing:
 	ackresponse=$($phpcli -f "$postrequest" "$url" "$action" "$gatewayhash" "$user_agent" "$acklist")
+	if [ $debuglevel -ge 3 ]; then
+		echo "authmon - remote FAS response [$ackresponse]" | logger -p "daemon.debug" -s -t "opennds[$ndspid]"
+	fi
 
 	# Sleep for a while:
 	sleep $loopinterval
