@@ -78,6 +78,7 @@ typedef enum {
 	oGatewayPort,
 	oGatewayFQDN,
 	oStatusPath,
+	oDhcpDefaultUrlEnable,
 	oFasPort,
 	oFasKey,
 	oFasPath,
@@ -150,6 +151,7 @@ static const struct {
 	{ "gatewayport", oGatewayPort },
 	{ "gatewayfqdn", oGatewayFQDN },
 	{ "statuspath", oStatusPath },
+	{ "dhcp_default_url_enable",oDhcpDefaultUrlEnable },
 	{ "fasport", oFasPort },
 	{ "faskey", oFasKey },
 	{ "fasremoteip", oFasRemoteIP },
@@ -240,6 +242,7 @@ config_init(void)
 	config.url_encoded_gw_name = NULL;
 	config.gw_fqdn = safe_strdup(DEFAULT_GATEWAYFQDN);
 	config.status_path = safe_strdup(DEFAULT_STATUSPATH);
+	config.dhcp_default_url_enable = DEFAULT_DHCP_DEFAULT_URL_ENABLE;
 	config.gw_interface = safe_strdup(DEFAULT_GATEWAYINTERFACE);;
 	config.gw_iprange = safe_strdup(DEFAULT_GATEWAY_IPRANGE);
 	config.gw_address = NULL;
@@ -830,6 +833,13 @@ config_read(const char *filename)
 		case oStatusPath:
 			config.status_path = safe_strdup(p1);
 			break;
+		case oDhcpDefaultUrlEnable:
+			if (sscanf(p1, "%d", &config.dhcp_default_url_enable) < 1) {
+				debug(LOG_ERR, "Bad arg %s to option %s on line %d in %s", p1, s, linenum, filename);
+				debug(LOG_ERR, "Exiting...");
+				exit(1);
+			}
+			break;
 		case oGatewayInterface:
 			config.gw_interface = safe_strdup(p1);
 			break;
@@ -1874,6 +1884,8 @@ void parse_fas_custom_files_list(const char ptr[])
 int set_debuglevel(const char opt[])
 {
 	char *end;
+	char *libcmd;
+	char msg[4] = {0};
 
 	if (opt == NULL || strlen(opt) == 0) {
 		return 1;
@@ -1887,6 +1899,16 @@ int set_debuglevel(const char opt[])
 
 	if (level >= DEBUGLEVEL_MIN && level <= DEBUGLEVEL_MAX) {
 		config.debuglevel = level;
+		safe_asprintf(&libcmd, "/usr/lib/opennds/libopennds.sh \"debuglevel\" \"%d\"", level);
+		debug(LOG_DEBUG, "Library command: [%s]", libcmd);
+
+		if (execute_ret_url_encoded(msg, sizeof(msg) - 1, libcmd) == 0) {
+			debug(LOG_NOTICE, "debuglevel [%d] signaled to externals - [%s] acknowledged", level, msg);
+		} else {
+			debug(LOG_ERR, "debuglevel [%d] signaled to externals - unable to set", level);
+		}
+
+		free(libcmd);
 		return 0;
 	} else {
 		return 1;
