@@ -29,7 +29,7 @@
 #ifndef _CONF_H_
 #define _CONF_H_
 
-#define VERSION "9.3.0"
+#define VERSION "9.4.0"
 
 /*
  * Defines how many times should we try detecting the interface with the default route (in seconds).
@@ -57,6 +57,7 @@
 #define DEFAULT_GATEWAYNAME "openNDS"
 #define DEFAULT_GATEWAYPORT 2050
 #define DEFAULT_GATEWAYFQDN "status.client"
+#define DEFAULT_DHCP_DEFAULT_URL_ENABLE 1
 #define DEFAULT_STATUSPATH "/usr/lib/opennds/client_params.sh"
 #define DEFAULT_LOG_MOUNTPOINT "/tmp"
 #define DEFAULT_MAX_PAGE_SIZE 10240
@@ -194,6 +195,7 @@ typedef struct {
 	char *gw_mac;					//@brief MAC address of the interface we manage
 	char *gw_fqdn;					//@brief FQDN of the client status page
 	char *status_path;				//@brief Path to the client status page script
+	int dhcp_default_url_enable;			//@brief Enable DHCP default-url (code 114 - RFC8910)
 	unsigned int gw_port;				//@brief Port the webserver will run on
 	unsigned int fas_port;				//@brief Port the fas server will run on
 	int login_option_enabled;			//@brief Use default PreAuth Login script
