@@ -416,8 +416,8 @@ enum MHD_Result libmicrohttpd_cb(
 
 	// only allow get
 	if (0 != strcmp(method, "GET")) {
-		debug(LOG_DEBUG, "Unsupported http method %s", method);
-		return send_error(connection, 403);
+		debug(LOG_DEBUG, "Unsupported http method %s, Network Authentication required (Error 511)", method);
+		return send_error(connection, 511);
 	}
 
 	// block path traversal
@@ -429,7 +429,7 @@ enum MHD_Result libmicrohttpd_cb(
 	// check for mhdstatus request
 	if (strstr(url, mhdstatus) != NULL) {
 		debug(LOG_DEBUG, "MHD Status Request - %s", url);
-		return send_error(connection, 511);
+		return send_error(connection, 200);
 	}
 
 
@@ -504,10 +504,6 @@ static int try_to_authenticate(struct MHD_Connection *connection, t_client *clie
 	char rhid[128] = {0};
 	char *rhidraw = NULL;
 
-	/* a successful auth looks like
-	 * http://192.168.42.1:2050/opennds_auth/?redir=http%3A%2F%2Fberlin.freifunk.net%2F&tok=94c4cdd2
-	 * when authaction -> http://192.168.42.1:2050/opennds_auth/
-	 */
 	config = config_get_config();
 
 	// Check for authdir
@@ -765,9 +761,9 @@ static int authenticated(struct MHD_Connection *connection,
 	}
 
 	// User just entered gatewayaddress:gatewayport so give them the info page
-	if (strcmp(url, "/") == 0) {
+	if (strcmp(url, "/") == 0 || strcmp(url, "/login") == 0) {
 		msg = safe_calloc(HTMLMAXSIZE);
-		rc = execute_ret(msg, HTMLMAXSIZE - 1, "%s '%s'", config->status_path, client->ip);
+		rc = execute_ret(msg, HTMLMAXSIZE - 1, "%s status '%s'", config->status_path, client->ip);
 
 		if (rc != 0) {
 			debug(LOG_WARNING, "Script: %s - failed to execute", config->status_path);
@@ -869,7 +865,6 @@ static int preauthenticated(struct MHD_Connection *connection,
 	char query_str[QUERYMAXLEN] = {0};
 	char *query = query_str;
 	char *querystr = query_str;
-	char portstr[MAX_HOSTPORTLEN] = {0};
 	char originurl[QUERYMAXLEN] = {0};
 
 	int ret;
@@ -912,26 +907,7 @@ static int preauthenticated(struct MHD_Connection *connection,
 		return send_error(connection, 511);
 	}
 
-
-	debug(LOG_DEBUG, "preauthenticated: Requested Host is [ %s ]", host);
-	debug(LOG_DEBUG, "preauthenticated: Requested url is [ %s ]", url);
-
-	// check if this is an attempt to directly access the basic splash page when FAS is enabled
-	if (config->fas_port) {
-		snprintf(portstr, MAX_HOSTPORTLEN, ":%u", config->gw_port);
-
-		debug(LOG_DEBUG, "preauthenticated: FAS is enabled");
-
-		if (check_authdir_match(url, config->authdir) || strstr(host, "/splash.css") == NULL) {
-			debug(LOG_DEBUG, "preauthenticated: splash.css or authdir detected");
-		} else {
-			if (strstr(host, portstr) != NULL) {
-				debug(LOG_DEBUG, "preauthenticated:  403 Direct Access Forbidden");
-				ret = send_error(connection, 403);
-				return ret;
-			}
-		}
-	}
+	debug(LOG_DEBUG, "preauthenticated: Requested Host is [ %s ], url is [%s]", host, url);
 
 	// check if this is a redirect query with a foreign host as target
 	if (is_foreign_hosts(connection, host)) {
@@ -939,9 +915,13 @@ static int preauthenticated(struct MHD_Connection *connection,
 		return redirect_to_splashpage(connection, client, host, url);
 	}
 
-	/* request is directed to us
-	 * check if client wants to be authenticated
-	 */
+	// check if this is an RFC8910 login request
+	if (strcmp(url, "/login") == 0) {
+		debug(LOG_DEBUG, "preauthenticated: RFC8910 login request detected - host [%s] url [%s]", host, url);
+		return redirect_to_splashpage(connection, client, host, "/");
+	}
+
+	// request is directed to us, check if client wants to be authenticated
 	if (check_authdir_match(url, config->authdir)) {
 		debug(LOG_DEBUG, "authdir url detected: %s", url);
 		redirect_url = get_redirect_url(connection);
@@ -1385,16 +1365,17 @@ static int send_error(struct MHD_Connection *connection, int error)
 	/* cannot automate since cannot translate automagically between error number and MHD's status codes
 	 * -- and cannot rely on MHD_HTTP_ values to provide an upper bound for an array
 	 */
-	const char *page_200 = "<html><header><title>Authenticated</title><body><h1>Authenticated</h1></body></html>";
+	const char *page_200 = "<br>OK<br>";
 	const char *page_400 = "<html><head><title>Error 400</title></head><body><h1>Error 400 - Bad Request</h1></body></html>";
 	const char *page_403 = "<html><head><title>Error 403</title></head><body><h1>Error 403 - Forbidden</h1></body></html>";
 	const char *page_404 = "<html><head><title>Error 404</title></head><body><h1>Error 404 - Not Found</h1></body></html>";
 	const char *page_500 = "<html><head><title>Error 500</title></head><body><h1>Error 500 - Internal Server Error. Oh no!</h1></body></html>";
 	const char *page_501 = "<html><head><title>Error 501</title></head><body><h1>Error 501 - Not Implemented</h1></body></html>";
 	const char *page_503 = "<html><head><title>Error 503</title></head><body><h1>Error 503 - Internal Server Error</h1></body></html>";
-	char *page_511 = NULL;
-	char *status_url = NULL;
+	char *page_511;
+	char *cmd = NULL;
 	const char *mimetype = lookup_mimetype("foo.html");
+	char ip[INET6_ADDRSTRLEN+1];
 
 	int ret = MHD_NO;
 	s_config *config = config_get_config();
@@ -1441,40 +1422,26 @@ static int send_error(struct MHD_Connection *connection, int error)
 		ret = MHD_queue_response(connection, MHD_HTTP_INTERNAL_SERVER_ERROR, response);
 		break;
 	case 511:
-		if (strcmp(config->gw_fqdn, "disable") != 0) {
-			safe_asprintf(&status_url, "http://%s/", config->gw_fqdn);
-		} else {
-			safe_asprintf(&status_url, "http://%s/", config->gw_ip);
+		get_client_ip(ip, connection);
+
+		page_511 = safe_calloc(HTMLMAXSIZE);
+		safe_asprintf(&cmd, "/usr/lib/opennds/client_params.sh err511 '%s'", ip);
+
+		if (execute_ret_url_encoded(page_511, HTMLMAXSIZE - 1, cmd) == 0) {
+			debug(LOG_INFO, "Network Authentication Required - page_511 html generated for [%s]", ip);
 		}
-		safe_asprintf(&page_511,
-			"<!DOCTYPE html>\n\
-			<html>\n\
-			<head>\n\
-			<meta http-equiv=\"Cache-Control\" content=\"no-cache, no-store, must-revalidate\">\n\
-			<meta http-equiv=\"Pragma\" content=\"no-cache\">\n\
-			<meta http-equiv=\"Expires\" content=\"0\">\n\
-			<meta charset=\"utf-8\">\n\
-			<meta name=\"viewport\" content=\"width=device-width, initial-scale=1.0\">\n\
-			<link rel=\"shortcut icon\" href=\"/images/splash.jpg\" type=\"image/x-icon\">\n\
-			<link rel=\"stylesheet\" type=\"text/css\" href=\"/splash.css\">\n\
-			<title>Error 511 Network Authentication Required</title></head>\n\
-			<body>\n\
-			<h1>Network Authentication Required</h1>\n\
-			<form action=\"%s\" method=\"get\">\n\
-			<input type=\"submit\" value=\"Refresh\">\n\
-			</form>\n\
-			<br>\n\
-			<form action=\"http://detectportal.firefox.com/success.txt\" method=\"get\" target=\"_blank\">\n\
-			<input type=\"submit\" value=\"Portal Login\" >\n\
-			</form>\n\
-			</body></html>\n",
-			status_url
-		);
+		free(cmd);
 
 		response = MHD_create_response_from_buffer(strlen(page_511), (char *)page_511, MHD_RESPMEM_MUST_FREE);
 		MHD_add_response_header(response, "Content-Type", mimetype);
 		ret = MHD_queue_response(connection, MHD_HTTP_NETWORK_AUTHENTICATION_REQUIRED, response);
-		free(status_url);
+
+		if (ret == MHD_NO) {
+			debug(LOG_ERR, "send_error 511: Error queueing response");
+		} else {
+			debug(LOG_DEBUG, "send_error 511: Response is Queued");
+		}
+
 		break;
 	}
 
@@ -1600,6 +1567,7 @@ static int serve_file(struct MHD_Connection *connection, t_client *client, const
 	ret = stat(filename, &stat_buf);
 	if (ret) {
 		// stat failed
+		debug(LOG_DEBUG, "File %s could not be found", filename);
 		return send_error(connection, 404);
 	}
 
