@@ -78,9 +78,9 @@
 #define MIN_MHD_MINOR 9
 #define MIN_MHD_PATCH 71
 
-/** XXX Ugly hack
- * We need to remember the thread IDs of threads that simulate wait with pthread_cond_timedwait
- * so we can explicitly kill them in the termination handler
+/** 
+ * Remember the thread IDs of threads that simulate wait with pthread_cond_timedwait
+ * in case we need them
  */
 static pthread_t tid_client_check = 0;
 
@@ -135,6 +135,10 @@ termination_handler(int s)
 {
 	static pthread_mutex_t sigterm_mutex = PTHREAD_MUTEX_INITIALIZER;
 	char *fasssl = NULL;
+	char *dnscmd;
+	char msg[256] = {0};
+	s_config *config;
+	config = config_get_config();
 
 	debug(LOG_INFO, "Handler for termination caught signal %d", s);
 
@@ -146,32 +150,41 @@ termination_handler(int s)
 		debug(LOG_INFO, "Cleaning up and exiting");
 	}
 
-	// Check if authmon is already running and if it is, kill it
-	debug(LOG_INFO, "Explicitly killing the authmon daemon");
-	safe_asprintf(&fasssl, "kill $(pgrep -f \"usr/lib/opennds/authmon.sh\") > /dev/null 2>&1");
+	// If authmon is running, kill it
+	if (config->fas_secure_enabled == 3) {
+		debug(LOG_INFO, "Explicitly killing the authmon daemon");
+		safe_asprintf(&fasssl, "kill $(pgrep -f \"usr/lib/opennds/authmon.sh\") > /dev/null 2>&1");
 
-	if (system(fasssl) < 0) {
-		debug(LOG_ERR, "Error returned from system call - Continuing");
+		if (system(fasssl) < 0) {
+			debug(LOG_ERR, "Error returned from system call - Continuing");
+		}
+
+		free(fasssl);
 	}
 
-	free(fasssl);
+	// If RFC8910 support is enabled, disable it
+	debug(LOG_DEBUG, "Disabling RFC8910 support");
+	safe_asprintf(&dnscmd, "/usr/lib/opennds/dnsconfig.sh \"cpidconf\"");
+
+	if (execute_ret_url_encoded(msg, sizeof(msg) - 1, dnscmd) == 0) {
+		debug(LOG_INFO, "RFC8910 support is disabled");
+	} else {
+		debug(LOG_ERR, "RFC8910 setup script failed to execute");
+	}
+	free(dnscmd);
+
+	// Restart dnsmasq
+	safe_asprintf(&dnscmd, "/usr/lib/opennds/dnsconfig.sh \"restart_only\" &");
+	debug(LOG_DEBUG, "restart command [ %s ]", dnscmd);
+	system(dnscmd);
+	debug(LOG_INFO, "Dnsmasq restarted");
+	free(dnscmd);
 
 	auth_client_deauth_all();
 
 	debug(LOG_INFO, "Flushing firewall rules...");
 	iptables_fw_destroy();
 
-	/* XXX Hack
-	 * Aparently pthread_cond_timedwait under openwrt prevents signals (and therefore
-	 * termination handler) from happening so we need to explicitly kill the threads
-	 * that use that
-	 */
-
-	if (tid_client_check) {
-		debug(LOG_INFO, "Explicitly killing the fw_counter thread");
-		pthread_kill(tid_client_check, SIGKILL);
-	}
-
 	debug(LOG_NOTICE, "Exiting...");
 	exit(s == 0 ? 1 : 0);
 }
@@ -422,84 +435,113 @@ setup_from_config(void)
 		debug(LOG_DEBUG, "Custom FAS files string [%s]", config->custom_files);
 	}
 
-	if (strcmp(config->gw_fqdn, "disable") != 0 || config->walledgarden_fqdn_list) {
-		// For Client status Page - configure the hosts file
-		if (strcmp(config->gw_fqdn, "disable") != 0) {
-			safe_asprintf(&dnscmd, "/usr/lib/opennds/dnsconfig.sh \"hostconf\" \"%s\" \"%s\"",
-				config->gw_ip,
-				config->gw_fqdn
-			);
+	// Do any required dnsmasq configurations and restart it
 
-			if (execute_ret_url_encoded(msg, sizeof(msg) - 1, dnscmd) == 0) {
-				debug(LOG_INFO, "Dnsmasq configured for Walled Garden");
-			} else {
-				debug(LOG_ERR, "Client Status Page: Hosts setup script failed to execute");
-			}
-			free(dnscmd);
+	// For Client status Page - configure the hosts file
+	if (strcmp(config->gw_fqdn, "disable") != 0 && strcmp(config->gw_fqdn, "disabled") != 0) {
+		safe_asprintf(&dnscmd, "/usr/lib/opennds/dnsconfig.sh \"hostconf\" \"%s\" \"%s\"",
+			config->gw_ip,
+			config->gw_fqdn
+		);
+
+		if (execute_ret_url_encoded(msg, sizeof(msg) - 1, dnscmd) == 0) {
+			debug(LOG_INFO, "Client status Page: Configured");
+		} else {
+			debug(LOG_ERR, "Client Status Page: Hosts setup script failed to execute");
 		}
-
-		// For Walled Garden - Check we have ipset support and if we do, set it up
-		if (config->walledgarden_fqdn_list) {
-			// Check ipset command is available
-			if (execute_ret_url_encoded(msg, sizeof(msg) - 1, "ipset -v") == 0) {
-				debug(LOG_NOTICE, "ipset support is available");
-			} else {
-				debug(LOG_ERR, "ipset support not available - please install package to provide it");
-				debug(LOG_ERR, "Exiting...");
-				exit(1);
-			}
-
-			// Check we have dnsmasq ipset compile option
-			if (execute_ret_url_encoded(msg, sizeof(msg) - 1, "dnsmasq --version | grep ' ipset '") == 0) {
-				debug(LOG_NOTICE, "dnsmasq ipset support is available");
-			} else {
-				debug(LOG_ERR, "Please install dnsmasq full version with ipset compile option");
-				debug(LOG_ERR, "Exiting...");
-				exit(1);
-			}
-
-			// If Walled Garden ipset exists, destroy it.
-			execute_ret_url_encoded(msg, sizeof(msg) - 1, "ipset destroy walledgarden");
-
-			// Set up the Walled Garden
-			if (execute_ret_url_encoded(msg, sizeof(msg) - 1, "ipset create walledgarden hash:ip") == 0) {
-				debug(LOG_INFO, "Walled Garden ipset created");
-			} else {
-				debug(LOG_ERR, "Failed to create Walled Garden");
-				debug(LOG_ERR, "Exiting...");
-				exit(1);
-			}
-
-			// Configure dnsmasq
-			for (allowed_wgfqdn = config->walledgarden_fqdn_list; allowed_wgfqdn != NULL; allowed_wgfqdn = allowed_wgfqdn->next) {
-
-				// Make sure we don't have a buffer overflow
-				if ((sizeof(wgfqdns) - strlen(wgfqdns)) > (strlen(allowed_wgfqdn->wgfqdn) + 15)) {
-					strcat(wgfqdns, "/");
-					strcat(wgfqdns, allowed_wgfqdn->wgfqdn);
-				} else {
-					break;
-				}
-			}
-
-			strcat(wgfqdns, "/walledgarden");
-			debug(LOG_DEBUG, "Dnsmasq Walled Garden config [%s]", wgfqdns);
-			safe_asprintf(&dnscmd, "/usr/lib/opennds/dnsconfig.sh \"ipsetconf\" \"%s\"", wgfqdns);
-			if (execute_ret_url_encoded(msg, sizeof(msg) - 1, dnscmd) == 0) {
-				debug(LOG_INFO, "Dnsmasq configured for Walled Garden");
-			} else {
-				debug(LOG_ERR, "Walled Garden Dnsmasq setup script failed to execute");
-			}
-			free(dnscmd);
-		}
-
-		safe_asprintf(&dnscmd, "/usr/lib/opennds/dnsconfig.sh \"restart_only\" &");
-		debug(LOG_DEBUG, "restart command [ %s ]", dnscmd);
-		system(dnscmd);
-		debug(LOG_INFO, "Dnsmasq restarted");
 		free(dnscmd);
 	}
 
+	// For Walled Garden - Check we have ipset support and if we do, set it up
+	if (config->walledgarden_fqdn_list) {
+		// Check ipset command is available
+		if (execute_ret_url_encoded(msg, sizeof(msg) - 1, "ipset -v") == 0) {
+			debug(LOG_NOTICE, "ipset support is available");
+		} else {
+			debug(LOG_ERR, "ipset support not available - please install package to provide it");
+			debug(LOG_ERR, "Exiting...");
+			exit(1);
+		}
+
+		// Check we have dnsmasq ipset compile option
+		if (execute_ret_url_encoded(msg, sizeof(msg) - 1, "dnsmasq --version | grep ' ipset '") == 0) {
+			debug(LOG_NOTICE, "dnsmasq ipset support is available");
+		} else {
+			debug(LOG_ERR, "Please install dnsmasq full version with ipset compile option");
+			debug(LOG_ERR, "Exiting...");
+			exit(1);
+		}
+
+		// If Walled Garden ipset exists, destroy it.
+		execute_ret_url_encoded(msg, sizeof(msg) - 1, "ipset destroy walledgarden");
+
+		// Set up the Walled Garden
+		if (execute_ret_url_encoded(msg, sizeof(msg) - 1, "ipset create walledgarden hash:ip") == 0) {
+			debug(LOG_INFO, "Walled Garden ipset created");
+		} else {
+			debug(LOG_ERR, "Failed to create Walled Garden");
+			debug(LOG_ERR, "Exiting...");
+			exit(1);
+		}
+
+		// Configure dnsmasq
+		for (allowed_wgfqdn = config->walledgarden_fqdn_list; allowed_wgfqdn != NULL; allowed_wgfqdn = allowed_wgfqdn->next) {
+
+			// Make sure we don't have a buffer overflow:
+			if ((sizeof(wgfqdns) - strlen(wgfqdns)) > (strlen(allowed_wgfqdn->wgfqdn) + 15)) {
+				strcat(wgfqdns, "/");
+				strcat(wgfqdns, allowed_wgfqdn->wgfqdn);
+			} else {
+				break;
+			}
+		}
+
+		strcat(wgfqdns, "/walledgarden");
+		debug(LOG_DEBUG, "Dnsmasq Walled Garden config [%s]", wgfqdns);
+		safe_asprintf(&dnscmd, "/usr/lib/opennds/dnsconfig.sh \"ipsetconf\" \"%s\"", wgfqdns);
+		if (execute_ret_url_encoded(msg, sizeof(msg) - 1, dnscmd) == 0) {
+			debug(LOG_INFO, "Dnsmasq configured for Walled Garden");
+		} else {
+			debug(LOG_ERR, "Walled Garden Dnsmasq setup script failed to execute");
+		}
+		free(dnscmd);
+	}
+
+	if (config->dhcp_default_url_enable == 1) {
+		debug(LOG_DEBUG, "Enabling RFC8910 support");
+
+		if (strcmp(config->gw_fqdn, "disable") != 0 && strcmp(config->gw_fqdn, "disabled") != 0) {
+			safe_asprintf(&dnscmd, "/usr/lib/opennds/dnsconfig.sh \"cpidconf\" \"%s\"", config->gw_fqdn);
+		} else {
+			safe_asprintf(&dnscmd, "/usr/lib/opennds/dnsconfig.sh \"cpidconf\" \"%s\"", config->gw_address);
+		}
+
+		if (execute_ret_url_encoded(msg, sizeof(msg) - 1, dnscmd) == 0) {
+			debug(LOG_INFO, "RFC8910 support is enabled");
+		} else {
+			debug(LOG_ERR, "RFC8910 setup script failed to execute");
+		}
+		free(dnscmd);
+	} else {
+		debug(LOG_DEBUG, "Disabling RFC8910 support");
+
+		safe_asprintf(&dnscmd, "/usr/lib/opennds/dnsconfig.sh \"cpidconf\"");
+
+		if (execute_ret_url_encoded(msg, sizeof(msg) - 1, dnscmd) == 0) {
+			debug(LOG_INFO, "RFC8910 support is disabled");
+		} else {
+			debug(LOG_ERR, "RFC8910 setup script failed to execute");
+		}
+		free(dnscmd);
+	}
+
+	// Restart dnsmasq
+	safe_asprintf(&dnscmd, "/usr/lib/opennds/dnsconfig.sh \"restart_only\" &");
+	debug(LOG_DEBUG, "restart command [ %s ]", dnscmd);
+	system(dnscmd);
+	debug(LOG_INFO, "Dnsmasq restarted");
+	free(dnscmd);
+
 	// Encode gatewayname
 	htmlentityencode(gw_name_entityencoded, sizeof(gw_name_entityencoded), config->gw_name, strlen(config->gw_name));
 	config->http_encoded_gw_name = safe_strdup(gw_name_entityencoded);
@@ -625,6 +667,11 @@ setup_from_config(void)
 				debug(LOG_DEBUG, "SSL Provider: %s FAS key is: %s\n", &msg, config->fas_key);
 			} else {
 				debug(LOG_ERR, "PHP packages PHP CLI and PHP OpenSSL are required");
+
+				if (config->fas_secure_enabled >= 3) {
+					debug(LOG_ERR, "Package ca-bundle is required for level 3 (https)");
+				}
+
 				debug(LOG_ERR, "Exiting...");
 				exit(1);
 			}
