@@ -91,33 +91,28 @@ thread_ndsctl(void *arg)
 	int number_of_count;
 	int i;
 
-	debug(LOG_DEBUG, "Starting ndsctl.");
+	debug(LOG_DEBUG, "Starting ndsctl thread");
 
 	memset(&sa_un, 0, sizeof(sa_un));
 	sock_name = (char *)arg;
 	debug(LOG_DEBUG, "Socket name: %s", sock_name);
 
 	if (strlen(sock_name) > (sizeof(sa_un.sun_path) - 1)) {
-		// TODO: Die handler with logging....
 		debug(LOG_ERR, "NDSCTL socket name too long");
 		exit(1);
 	}
 
-	debug(LOG_DEBUG, "Creating socket");
-	sock = socket(PF_UNIX, SOCK_STREAM, 0);
+	// Use AF_UNIX, not PF_UNIX, AF_LOCAL or PF_LOCAL
+	sock = socket(AF_UNIX, SOCK_STREAM, 0);
 
-	debug(LOG_DEBUG, "Got server socket %d", sock);
-
-	// If it exists, delete... Not the cleanest way to deal.
+	// If socket file exists, delete it..
 	unlink(sock_name);
 
-	debug(LOG_DEBUG, "Filling sockaddr_un");
-	strcpy(sa_un.sun_path, sock_name); // XXX No size check because we check a few lines before.
+	strcpy(sa_un.sun_path, sock_name);
 	sa_un.sun_family = AF_UNIX;
 
-	debug(LOG_DEBUG, "Binding socket (%s) (%d)", sa_un.sun_path, strlen(sock_name));
+	debug(LOG_DEBUG, "Binding socket [%s] Socket descriptor [%d]", sa_un.sun_path, sock);
 
-	// Which to use, AF_UNIX, PF_UNIX, AF_LOCAL, PF_LOCAL?
 	if (bind(sock, (struct sockaddr *)&sa_un, strlen(sock_name) + sizeof(sa_un.sun_family))) {
 		debug(LOG_ERR, "Could not bind control socket: [%s] Terminating...", strerror(errno));
 		pthread_exit(NULL);
@@ -148,6 +143,8 @@ thread_ndsctl(void *arg)
 
 	current_fd_count = 1;
 
+	debug(LOG_DEBUG, "Entering ndsctl thread loop");
+
 	while (1) {
 		memset(&sa_un, 0, sizeof(sa_un));
 		len = (socklen_t) sizeof(sa_un);
@@ -209,7 +206,6 @@ thread_ndsctl(void *arg)
 				if (events[i].data.fd > 0) {
 					events[i].data.fd = 0;
 				}
-
 			}
 		}
 	}
@@ -227,7 +223,6 @@ ndsctl_handler(int fd)
 	FILE* fp;
 
 	debug(LOG_DEBUG, "Entering thread_ndsctl_handler....");
-	debug(LOG_DEBUG, "Read bytes and stuff from descriptor %d", fd);
 
 	// Init variables
 	read_bytes = 0;
@@ -235,6 +230,8 @@ ndsctl_handler(int fd)
 	memset(request, 0, sizeof(request));
 	fp = fdopen(fd, "w");
 
+	debug(LOG_DEBUG, "Read bytes and stuff from socket descriptor [%d], pointer [%lu]", fd, fp);
+
 	// Read....
 	while (!done && read_bytes < (sizeof(request) - 1)) {
 		len = read(fd, request + read_bytes, sizeof(request) - read_bytes);
@@ -305,6 +302,8 @@ ndsctl_auth(FILE *fp, char *arg)
 	int downloadrate = config->download_rate;
 	unsigned long long int uploadquota = config->upload_quota;
 	unsigned long long int downloadquota = config->download_quota;
+	char *testcmd;
+	char msg[64] = {0};
 	char customdata[256] = {0};
 	char *argcopy;
 	const char *arg2;
@@ -315,6 +314,8 @@ ndsctl_auth(FILE *fp, char *arg)
 	const char *arg7;
 	const char *arg8;
 	char *ptr;
+	char *ipclient;
+	char *macclient;
 	time_t now = time(NULL);
 
 	debug(LOG_DEBUG, "Entering ndsctl_auth [%s]", arg);
@@ -378,11 +379,43 @@ ndsctl_auth(FILE *fp, char *arg)
 	}
 
 	LOCK_CLIENT_LIST();
-	debug(LOG_DEBUG, "find in client list - arg2: [%d]", arg2);
+	debug(LOG_DEBUG, "find in client list - arg2: [%s]", arg2);
 	client = client_list_find_by_any(arg2, arg2, arg2);
 	id = client ? client->id : 0;
 	debug(LOG_DEBUG, "client id: [%d]", id);
 
+	if (!id  && config->allow_preemptive_authentication == 1) {
+		// Client is neither preauthenticated nor authenticated
+		// If Preemptive authentication is enabled we should try to auth by mac
+		debug(LOG_DEBUG, "Client is not in client list.");
+		// Build command to get client mac and ip
+		safe_asprintf(&testcmd,
+			"/usr/lib/opennds/libopennds.sh clientaddress \"%s\"",
+			arg2
+		);
+
+		if (execute_ret_url_encoded(msg, sizeof(msg) - 1, testcmd) == 0) {
+			debug(LOG_DEBUG, "Client ip/mac: %s", msg);
+
+			if (strcmp(msg, "-") == 0) {
+				debug(LOG_DEBUG, "Client [%s] is not connected", arg2);
+			} else {
+				ipclient = strtok(msg, " ");
+				macclient = strtok(NULL, " ");
+				debug(LOG_DEBUG, "Client ip [%s], mac [%s]", ipclient, macclient);
+
+				client_list_add_client(macclient, ipclient);
+				client = client_list_find_by_any(arg2, arg2, arg2);
+				id = client ? client->id : 0;
+				debug(LOG_DEBUG, "client id: [%d]", id);
+			}
+		} else {
+			debug(LOG_DEBUG, "Client connection not found: Continuing...");
+			rc = -1;
+		}
+		free(testcmd);
+	}
+
 	if (id) {
 
 		if (strcmp(fw_connection_state_as_string(client->fw_connection_state), "Preauthenticated") == 0) {
@@ -405,7 +438,9 @@ ndsctl_auth(FILE *fp, char *arg)
 			rc = auth_client_auth_nolock(id, "ndsctl_auth", customdata);
 		}
 	} else {
-		debug(LOG_DEBUG, "Client not found.");
+		// Client is neither preauthenticated nor authenticated
+		// If Preemptive authentication is enabled we should try to auth by mac
+		debug(LOG_DEBUG, "Client is not in client list.");
 		rc = -1;
 	}
 
