@@ -302,7 +302,9 @@ static int _execute_ret(char* msg, int msg_len, const char *cmd)
 	}
 
 	if (msg && msg_len > 0) {
+		debug(LOG_DEBUG, "Reading command output");
 		rc = fread(msg, msg_len - 1, 1, fp);
+		debug(LOG_DEBUG, "command output: [%s]", msg);
 	}
 
 	rc = pclose(fp);
@@ -558,6 +560,7 @@ ndsctl_status(FILE *fp)
 
 	fprintf(fp, "Uptime: %s\n", durationbuf);
 	fprintf(fp, "Gateway Name: [ %s ]\n", config->gw_name);
+	fprintf(fp, "Debug Level: [ %d ]\n", config->debuglevel);
 	fprintf(fp, "Gateway FQDN: [ %s ]\n", config->gw_fqdn);
 
 	if (strstr(config->gw_iprange, "0.0.0.0/0")) {
@@ -810,6 +813,8 @@ ndsctl_json_client(FILE *fp, const t_client *client, time_t now, char *indent)
 	get_client_interface(clientif, sizeof(clientif), client->mac);
 
 	fprintf(fp, "  %s\"gatewayname\":\"%s\",\n", indent, config->url_encoded_gw_name);
+	fprintf(fp, "  %s\"gatewayaddress\":\"%s\",\n", indent, config->gw_address);
+	fprintf(fp, "  %s\"gatewayfqdn\":\"%s\",\n", indent, config->gw_fqdn);
 	fprintf(fp, "  %s\"version\":\"%s\",\n", indent, VERSION);
 	fprintf(fp, "  %s\"mac\":\"%s\",\n", indent, client->mac);
 	fprintf(fp, "  %s\"ip\":\"%s\",\n", indent, client->ip);
